// ServerSocket.cpp : implementation file
//

#include "stdafx.h"
#include "CodeWarHost_1.h"
#include "ServerSocket.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CServerSocket

CServerSocket::CServerSocket(basic_arena &a):arena(a)
{
    state = login;
}

CServerSocket::~CServerSocket()
{
    for(int i=0; i<clients.size(); ++i){
        delete clients[i];
    }
}


// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CServerSocket, CSocket)
	//{{AFX_MSG_MAP(CServerSocket)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

/////////////////////////////////////////////////////////////////////////////
// CServerSocket member functions

void CServerSocket::OnAccept(int nErrorCode) 
{
    static short client_id = 0;
    CClientSocket * client;
    CString msg;
    DWORD err;
    switch(state){
    case login:
        client = new CClientSocket(arena, this);
        if(Accept(dynamic_cast<CAsyncSocket &>(*client))){
            client->s_handshake.species_id = client_id;
            client->s_handshake.no_of_creatures = war_parameters.no_of_creatures;
            ++client_id;
            clients.push_back(client);
            std::cout<<"Connected to client "<<client->s_handshake.species_id<<std::endl;
            msg.Format("Connected to client %d", client->s_handshake.species_id);
            DisplayLine(msg);
            codewar::server_handshake_message message(client->s_handshake);
            if(!client->Send(&message, sizeof(message))){
                DisplayLine("Cannot send server handshake to xxx");
            }
        }
        else{
            delete client;
            err = GetLastError();
            std::cout<<"Cannot connect to client "<<client_id<<", (error "<<err<<") "<<__FILE__<<": "<<__LINE__<<std::endl;
            msg.Format("Cannot connect to client %d", client_id);
            DisplayLine(msg);
        }
        break;
    case war:
        // do not accept new connections
        break;
    default:
        ASSERT(0);
    }
	CSocket::OnAccept(nErrorCode);
}

void CServerSocket::GetSightedCreatureInfo(codewar::sighted_creature_info &creature_info, short species, short id)
{
    CClientSocket::creature real_creature = clients[species]->creatures[id];
    creature_info.species = species;
    creature_info.id = id;
    creature_info.life = (100*real_creature.data.life)/clients[species]->stats.max_life;
    creature_info.facing = real_creature.data.facing;
}

void CServerSocket::MakeTurn(bool force/*not used*/)
{
    static int turn_no = 0;
    if(turn_no == war_parameters.no_of_turns){
        DisplayLine("Game over.");
        return;
    }
    bool turn = true;
    for(int i=0; i<clients.size(); ++i){
        for(int j=0; j<clients[i]->creatures.size();++j){
            turn = turn && (clients[i]->creatures[j].next_turn_action != codewar::idle);
        }
    }
    if(turn){
        ///////////////////////////////
        //
        // compute turn result : not implemented 
        //
        ///////////////////////////////
        TRACE("Turn = %d\n", turn_no);
        std::cout<<"Turn "<<turn_no<<std::endl;
        for(int i=0; i<clients.size(); ++i){
            clients[i]->SendFeedback();    
        }
        Sleep(10); // needed -- or else it will hang (?!)
        ++turn_no;
    }
}

void CServerSocket::RandomizeCreatureLocations()
{
    static unsigned seed = (unsigned)time(NULL);
    srand(seed);
    seed+= 1234;
    for(int i=0; i<clients.size(); ++i){
        for(int j=0; j<war_parameters.no_of_creatures; ++j){
            CClientSocket::creature c;
            do{
                c.data.x = rand()%(arena.cells[0].size()-2) + 1;
                c.data.y = rand()%(arena.cells.size()-2) + 1;
            } while(!arena.cells[c.data.x][c.data.y].contents == codewar::cell_empty);
            arena.cells[c.data.x][c.data.y].contents = codewar::cell_creature;
            arena.cells[c.data.x][c.data.y].creature = j;
            arena.cells[c.data.x][c.data.y].species = i;
            c.data.facing = rand()%4;
            c.data.id = j;
            c.data.life = clients[i]->stats.max_life;
            c.data.food = war_parameters.starting_food;
            c.data.species = i;
            c.last_turn_result = codewar::result_ok;
            c.next_turn_action = codewar::idle;
            clients[i]->creatures.push_back(c);
            std::cout<<"Randomizing location for species "<<i<<", creature "<<j<<std::endl;
            std::cout<<"x = "<<c.data.x<<", y = "<<c.data.y<<", facing = "<<c.data.facing<<std::endl;
        }
    }
}

void CServerSocket::SendFeedbackToClients()
{
    for(int i=0; i<clients.size(); ++i){
        clients[i]->SendFeedback();
    }
}
